@@ -1,34 +1,38 @@
-import typing
-from typing import Dict
-
 import ccxt
 import pandas as pd
 from quick_trade import utils
+from typing import Dict
 
 
 class TradingClient(object):
     ordered: bool = False
     __side__: str
-    quantity: float
     ticker: str
-    order: Dict[str, typing.Any]
+    cls_open_orders: int = 0
+    start_balance: Dict[str, float]
+    base: str
+    quote: str
 
     def __init__(self, client: ccxt.Exchange):
         self.client = client
+        self._update_balances()
 
     def order_create(self,
                      side: str,
                      ticker: str = 'None',
                      quantity: float = 0.0):
+        self._update_balances()
         utils.logger.info(f'quantity: {quantity}, side: {side}')
         if side == 'Buy':
-            self.order = self.client.create_market_buy_order(symbol=ticker, amount=quantity)
+            self.client.create_market_buy_order(symbol=ticker, amount=quantity)
         elif side == 'Sell':
-            self.order = self.client.create_market_sell_order(symbol=ticker, amount=quantity)
-        self.quantity = quantity
+            self.client.create_market_sell_order(symbol=ticker, amount=quantity)
         self.__side__ = side
         self.ticker = ticker
+        self.base = ticker.split('/')[0]
+        self.quote = ticker.split('/')[1]
         self.ordered = True
+        self._add_order_count()
 
     def get_ticker_price(self,
                          ticker: str) -> float:
@@ -70,15 +74,31 @@ class TradingClient(object):
 
     def exit_last_order(self):
         if self.ordered:
+            base_balance = self.get_balance_ticker(self.base)
+            bet = base_balance - self.start_balance[self.base]
             if self.__side__ == 'Sell':
-                self.new_order_buy(self.ticker, self.get_balance_ticker(self.ticker.split('/')[0]),
-                                   logging=False)  # buy for all balance
+                self.new_order_buy(self.ticker,
+                                   bet,
+                                   logging=False)
             elif self.__side__ == 'Buy':
-                self.new_order_sell(self.ticker, self.get_balance_ticker(self.ticker.split('/')[1]),
-                                    logging=False)  # sell all
+                self.new_order_sell(self.ticker,
+                                    bet,
+                                    logging=False)
             self.__side__ = 'Exit'
             self.ordered = False
             utils.logger.info('client exit')
+            self._sub_order_count()
 
     def get_balance_ticker(self, ticker: str) -> float:
         return self.client.fetch_free_balance()[ticker]
+
+    @classmethod
+    def _add_order_count(cls):
+        cls.cls_open_orders += 1
+
+    @classmethod
+    def _sub_order_count(cls):
+        cls.cls_open_orders -= 1
+
+    def _update_balances(self):
+        self.start_balance =self.client.fetch_free_balance()
