@@ -1,21 +1,14 @@
-"""
-Trading project:
-    - testing
-    - trading
-    - quick_trade-tuner
-
-"""
-
 # !/usr/bin/python
 # -*- coding: utf-8 -*-
 # used ta by Darío López Padial (Bukosabino https://github.com/bukosabino/ta)
 
 
 # TODO:
-#   add inner class with non-trading utils
 #   add tradingview realtime signals
 #   numpy
 #   scalper and dca bot
+#   unit-tests
+#   more docs and examples
 
 import time
 import typing
@@ -31,10 +24,11 @@ import ta.trend
 import ta.volatility
 import ta.volume
 import talib
-from plotly.graph_objs import Line
 from plotly.subplots import make_subplots
 from quick_trade import utils
 
+Line = dict  # To avoid the deprecation warning
+
 
 class Trader(object):
     """
@@ -84,70 +78,12 @@ class Trader(object):
                  ticker: str = 'BTC/USDT',
                  df: pd.DataFrame = pd.DataFrame(),
                  interval: str = '1d',
-                 trading_on_client: bool = False):
+                 trading_on_client: bool = True):
         self.df = df.reset_index(drop=True)
         self.ticker = ticker
         self.interval = interval
         self.trading_on_client = trading_on_client
-        if interval == '1m':
-            self._profit_calculate_coef = 1 / (60 * 24 * 365)
-            self._sec_interval = 60
-        elif interval == '2m':
-            self._profit_calculate_coef = 1 / (30 * 24 * 365)
-            self._sec_interval = 120
-        elif interval == '3m':
-            self._profit_calculate_coef = 1 / (20 * 24 * 365)
-            self._sec_interval = 180
-        elif interval == '5m':
-            self._profit_calculate_coef = 1 / (12 * 24 * 365)
-            self._sec_interval = 300
-        elif interval == '15m':
-            self._profit_calculate_coef = 1 / (4 * 24 * 365)
-            self._sec_interval = 15 * 60
-        elif interval == '30m':
-            self._profit_calculate_coef = 1 / (2 * 24 * 365)
-            self._sec_interval = 60 * 30
-        elif interval == '45m':
-            self._profit_calculate_coef = 1 / (32 * 365)
-            self._sec_interval = 60 * 45
-        elif interval == '1h':
-            self._profit_calculate_coef = 1 / (24 * 365)
-            self._sec_interval = 60 * 60
-        elif interval == '90m':
-            self._profit_calculate_coef = 1 / (18 * 365)
-            self._sec_interval = 60 * 90
-        elif interval == '2h':
-            self._profit_calculate_coef = 1 / (12 * 365)
-            self._sec_interval = 60 * 60 * 2
-        elif interval == '3h':
-            self._profit_calculate_coef = 1 / (8 * 365)
-            self._sec_interval = 60 * 60 * 3
-        elif interval == '4h':
-            self._profit_calculate_coef = 1 / (6 * 365)
-            self._sec_interval = 60 * 60 * 4
-        elif interval == '12h':
-            self._profit_calculate_coef = 1 / (2 * 365)
-            self._sec_interval = 60 * 60 * 12
-        elif interval == '1d':
-            self._profit_calculate_coef = 1 / 365
-            self._sec_interval = 60 * 60 * 24
-        elif interval == '3d':
-            self._profit_calculate_coef = 1 / (365 / 3)
-            self._sec_interval = 86400 * 3
-        elif interval == '1w':
-            self._profit_calculate_coef = 1 / 52
-            self._sec_interval = 86400 * 7
-        elif interval == '1M':
-            self._profit_calculate_coef = 1 / 12
-            self._sec_interval = 86400 * 30
-        elif interval == '3M':
-            self._profit_calculate_coef = 1 / 4
-            self._sec_interval = 86400 * 90
-        elif interval == '6M':
-            self._profit_calculate_coef = 1 / 2
-            self._sec_interval = 86400 * 180
-        else:
-            raise ValueError(f'incorrect interval; {interval}')
+        self._profit_calculate_coef, self._sec_interval = utils.get_coef_sec(interval)
         self.__exit_order__ = False
 
     def __repr__(self):
@@ -163,7 +99,7 @@ class Trader(object):
     def __get_stop_take(self, sig: utils.PREDICT_TYPE) -> Dict[str, float]:
         """
         calculating stop loss and take profit.
-        sig:        |     int     |  signal to sell/buy/exit:
+        sig:        |     PREDICT_TYPE     |  signal to sell/buy/exit:
             EXIT -- exit.
             BUY -- buy.
             SELL -- sell.
@@ -208,7 +144,7 @@ class Trader(object):
         take_profits = []
         for stop_loss_price, take_profit_price, price, sig in zip(self._stop_losses,
                                                                   self._take_profits,
-                                                                  self.df['Close'].values,
+                                                                  self._open_lot_prices,
                                                                   self.returns):
             add_sl = (price / 10_000) * add_stop_loss
             add_tp = (price / 10_000) * add_take_profit
@@ -227,7 +163,7 @@ class Trader(object):
         self._take_profits = take_profits
         return self._stop_losses, self._take_profits
 
-    def strategy_diff(self, frame_to_diff: pd.Series, *args) -> utils.PREDICT_TYPE_LIST:
+    def strategy_diff(self, frame_to_diff: pd.Series) -> utils.PREDICT_TYPE_LIST:
         """
         frame_to_diff:  |   pd.Series  |  example:  Trader.df['Close']
         """
@@ -401,7 +337,7 @@ class Trader(object):
             else:
                 self.returns.append(utils.EXIT)
         self.set_credit_leverages()
-        self.set_open_stop_and_take()
+        self.set_open_stop_and_take(set_stop=False)
         return self.returns
 
     def strategy_macd_histogram_diff(self,
@@ -574,8 +510,6 @@ class Trader(object):
                 max_cloud = max((A, B))
                 min_cloud = min((A, B))
 
-                stop_loss_adder = stop_loss_plus * (close / 10_000)
-
                 if not min_cloud < close < max_cloud:
                     if tenkan > kijun:
                         flag1 = utils.BUY
@@ -597,18 +531,16 @@ class Trader(object):
                     if (trade == utils.BUY and flag1 == utils.SELL) or (trade == utils.SELL and flag1 == utils.BUY):
                         trade = utils.EXIT
                 self.returns.append(trade)
-                if trade == utils.BUY:
-                    self._stop_losses.append(min_cloud - stop_loss_adder)
-                else:
-                    self._stop_losses.append(max_cloud + stop_loss_adder)
+
         self.set_open_stop_and_take(set_take=True,
                                     set_stop=False)
         self.set_credit_leverages()
+        self.sl_tp_adder(add_stop_loss=stop_loss_plus)
         return self.returns
 
     def crossover(self, fast: Iterable, slow: Iterable):
         self.returns = []
-        for s, f in zip(list(slow), list(fast)):
+        for s, f in zip(slow, fast):
             if s < f:
                 self.returns.append(utils.BUY)
             elif s > f:
@@ -726,14 +658,12 @@ class Trader(object):
                 exit_take_stop = False
                 ignore_breakout = True
 
-            next_breakout = min(stop_loss, take_profit) < next_l <= next_h < max(stop_loss, take_profit)
+            next_not_breakout = min(stop_loss, take_profit) < next_l <= next_h < max(stop_loss, take_profit)
             if (min(stop_loss, take_profit) < low <= high < max(stop_loss,
-                                                                take_profit) and next_breakout) or ignore_breakout:
+                                                                take_profit) and next_not_breakout) or ignore_breakout:
                 diff = data_column[e + 1] - data_column[e]
             else:
                 exit_take_stop = True
-                if next_breakout:
-                    e += 1
                 if sig == utils.BUY and high >= take_profit:
                     diff = take_profit - data_column[e]
 
@@ -748,8 +678,6 @@ class Trader(object):
 
                 else:
                     diff = 0.0
-                if next_breakout:
-                    e -= 1
 
             if sig == utils.SELL:
                 diff = -diff
@@ -773,7 +701,7 @@ class Trader(object):
         self._linear = utils.get_linear(self.deposit_history)
         lin_calc_df = pd.DataFrame(self._linear)
         mean_diff = float(lin_calc_df.diff().mean())
-        self.year_profit = mean_diff / self._profit_calculate_coef
+        self.year_profit = mean_diff * self._profit_calculate_coef
         self.year_profit = (self.year_profit / money_start) * 100
         if self.trades != 0:
             self.winrate = (self.profits / self.trades) * 100
@@ -892,7 +820,7 @@ winrate: {self.winrate}%"""
         return self.backtest_out
 
     def multi_backtest(self,
-                       tickers: Iterable[str],
+                       tickers: typing.Union[typing.Sized, typing.Iterable[str]],
                        strategy_name: str = 'strategy_macd',
                        strategy_kwargs: Dict[str, Any] = {},
                        deposit: float = 10_000.0,
@@ -908,7 +836,8 @@ winrate: {self.winrate}%"""
         losses: List[int] = []
         trades: List[int] = []
         profits: List[int] = []
-        depo: List[List[float]] = []
+        depo: List[np.ndarray] = []
+        lens_dep: List[int] = []
 
         for ticker in tickers:
             df = self.client.get_data_historical(ticker=ticker, limit=limit, interval=self.interval)
@@ -928,13 +857,18 @@ winrate: {self.winrate}%"""
             losses.append(new_trader.losses)
             trades.append(new_trader.trades)
             profits.append(new_trader.profits)
-            depo.append(new_trader.deposit_history)
+            depo.append(np.array(new_trader.deposit_history))
+            lens_dep.append(len(new_trader.deposit_history))
         self.losses = sum(losses)
         self.trades = sum(profits)
         self.profits = sum(profits)
         self.year_profit = float(np.mean(percentage_profits))
         self.winrate = float(np.mean(winrates))
-        self.deposit_history = list(sum(np.array(depo)))
+
+        for enum, elem in enumerate(depo):
+            depo[enum] = np.array(elem[-min(lens_dep):]) / (elem[-min(lens_dep)] / (deposit / len(tickers)))
+        self.deposit_history = list(sum(depo))
+
         self._linear = utils.get_linear(self.deposit_history)
         self.returns_strategy_diff = list(pd.Series(self.deposit_history).diff().values)
         self.returns_strategy_diff[0] = 0
@@ -980,11 +914,9 @@ winrate: {self.winrate}%"""
                    width: int = 1300,
                    template: str = 'plotly_dark',
                    row_heights: list = [10, 16, 7],
-                   quick_trade_logo: bool = True,
                    **subplot_kwargs):
         """
 
-        :param quick_trade_logo: use quick trade logo as background
         :param height: window height
         :param width: window width
         :param template: plotly template
@@ -1055,7 +987,7 @@ winrate: {self.winrate}%"""
 
                     ...
 
-                    first_returns = [1,1,0,0,2,0,2,2,0,0,1]
+                    first_returns =  [1,1,0,0,2,0,2,2,0,0,1]
 
                     second_returns = [1,2,2,2,2,2,0,0,0,0,1]
 
@@ -1064,7 +996,7 @@ winrate: {self.winrate}%"""
                 mode = 'super':
                     ...
 
-                    first_returns = [1,1,1,2,2,2,0,0,1]
+                    first_returns =  [1,1,1,2,2,2,0,0,1]
 
                     second_returns = [1,0,0,0,1,1,1,0,0]
 
@@ -1081,16 +1013,16 @@ winrate: {self.winrate}%"""
                 else:
                     self.returns.append(utils.EXIT)
         elif mode == 'maximalist':
-            self.returns = self.__maximalist(first_returns, second_returns)
+            self.returns = self._maximalist(first_returns, second_returns)
         elif mode == 'super':
-            self.returns = self.__collide_super(first_returns, second_returns)
+            self.returns = self._collide_super(first_returns, second_returns)
         else:
             raise ValueError('incorrect mode')
         return self.returns
 
     @staticmethod
-    def __maximalist(returns1: utils.PREDICT_TYPE_LIST,
-                     returns2: utils.PREDICT_TYPE_LIST) -> utils.PREDICT_TYPE_LIST:
+    def _maximalist(returns1: utils.PREDICT_TYPE_LIST,
+                    returns2: utils.PREDICT_TYPE_LIST) -> utils.PREDICT_TYPE_LIST:
         return_list: utils.PREDICT_TYPE_LIST = []
         flag = utils.EXIT
         for a, b in zip(returns1, returns2):
@@ -1102,7 +1034,7 @@ winrate: {self.winrate}%"""
         return return_list
 
     @staticmethod
-    def __collide_super(l1, l2) -> utils.PREDICT_TYPE_LIST:
+    def _collide_super(l1, l2) -> utils.PREDICT_TYPE_LIST:
         return_list: utils.PREDICT_TYPE_LIST = []
         for first, sec in zip(utils.set_(l1), utils.set_(l2)):
             if first is not np.nan and sec is not np.nan and first is not sec:
@@ -1113,7 +1045,7 @@ winrate: {self.winrate}%"""
                 return_list.append(sec)
             else:
                 return_list.append(first)
-        return utils.anti_set_(return_list)
+        return list(map(lambda x: utils.PREDICT_TYPE(x), utils.anti_set_(return_list)))
 
     def multi_strategy_collider(self, *strategies, mode: str = 'minimalist') -> utils.PREDICT_TYPE_LIST:
         self.strategy_collider(strategies[0], strategies[1], mode=mode)
@@ -1126,7 +1058,7 @@ winrate: {self.winrate}%"""
 
     def get_trading_predict(self,
                             bet_for_trading_on_client: float = np.inf,
-                            coin_lotsize_division=True
+                            coin_lotsize_division: bool = True
                             ) -> Dict[str, typing.Union[str, float]]:
         """
         predict and trading.
@@ -1136,19 +1068,12 @@ winrate: {self.winrate}%"""
         :return: dict with prediction
         """
 
-        credit_leverage: float = self._credit_leverages[-1]
         _moneys_: float
         bet: float
         close: np.ndarray = self.df["Close"].values
-        cond: bool
 
         # get prediction
-        predict = self.returns[-1]
-        predict = utils.convert_signal_str(predict)
-        if self.__exit_order__ and self._prev_predict == predict:
-            predict = 'Exit'
-        if predict != 'Exit':
-            self.__exit_order__ = False
+        predict = utils.convert_signal_str(self.returns[-1])
 
         # trading
         self.__last_stop_loss = self._stop_losses[-1]
@@ -1156,29 +1081,30 @@ winrate: {self.winrate}%"""
         self.__last_credit_leverage = self._credit_leverages[-1]
         if self._prev_predict != predict or self.__prev_credit_lev != self.__last_credit_leverage:
             utils.logger.info(f'open lot {predict}')
-            self._open_price = close[-1]
+            self.__exit_order__ = False
             if self.trading_on_client:
 
                 if predict == 'Exit':
                     self.client.exit_last_order()
+                    self.__exit_order__ = True
 
                 else:
                     _moneys_ = self.client.get_balance_ticker(self.ticker.split('/')[1])
                     ticker_price = self.client.get_ticker_price(self.ticker)
-                    if coin_lotsize_division:
-                        _moneys_ /= ticker_price
                     if bet_for_trading_on_client is not np.inf:
-                        bet = bet_for_trading_on_client / ticker_price
+                        bet = bet_for_trading_on_client
                     else:
                         bet = _moneys_
                     if bet > _moneys_:
                         bet = _moneys_
+                    if coin_lotsize_division:
+                        bet /= ticker_price
                     self.client.exit_last_order()
 
                     self.client.order_create(predict,
                                              self.ticker,
-                                             bet * credit_leverage)
-                    self.__exit_order__ = False
+                                             bet * self.__last_credit_leverage)
+        utils.logger.debug("returning prediction")
         return {
             'predict': predict,
             'open lot price': self._open_price,
@@ -1202,7 +1128,7 @@ winrate: {self.winrate}%"""
                          *strategy_args,
                          **strategy_kwargs):
         """
-        :param strategy_in_sleep: reuse strategy in candle for new S/L, T/P or martingale
+        :param strategy_in_sleep: reuse strategy in one candle for new S/L, T/P or martingale
         :param limit: client.get_data_historical's limit argument
         :param wait_sl_tp_checking: sleeping time after stop-loss and take-profit checking (seconds)
         :param print_exc: print  exceptions in while loop
@@ -1218,11 +1144,14 @@ winrate: {self.winrate}%"""
 
         self.realtime_returns = {}
         self.ticker = ticker
-        __now__ = time.time()
+        open_time = time.time()
         while True:
             try:
                 self.df = self.client.get_data_historical(ticker=self.ticker, limit=limit, interval=self.interval)
+                utils.logger.debug("new dataframe loaded")
+
                 strategy(*strategy_args, **strategy_kwargs)
+                utils.logger.debug("strategy used")
 
                 prediction = self.get_trading_predict(
                     bet_for_trading_on_client=bet_for_trading_on_client,
@@ -1235,14 +1164,16 @@ winrate: {self.winrate}%"""
                 self.realtime_returns[index] = prediction
                 while True:
                     if not self.__exit_order__:
+                        if (open_time + self._sec_interval) - time.time() < wait_sl_tp_checking:
+                            time.sleep(wait_sl_tp_checking)
+                        utils.logger.debug(f"sleep {wait_sl_tp_checking} seconds")
+
                         price = self.client.get_ticker_price(ticker)
                         min_ = min(self.__last_stop_loss, self.__last_take_profit)
                         max_ = max(self.__last_stop_loss, self.__last_take_profit)
                         if (not (min_ < price < max_)) and prediction["predict"] != 'Exit':
                             self.__exit_order__ = True
                             utils.logger.info('exit lot')
-                            prediction['predict'] = 'Exit'
-                            prediction['currency close'] = price
                             index = f'{self.ticker}, {time.ctime()}'
                             utils.logger.info(f"trading prediction exit in sleeping at {index}: {prediction}")
                             if print_out:
@@ -1252,12 +1183,12 @@ winrate: {self.winrate}%"""
                                 self.client.exit_last_order()
                         elif strategy_in_sleep:
                             break
-                    time.sleep(wait_sl_tp_checking)
-                    if not (time.time() < (__now__ + self._sec_interval)):
+                    if time.time() >= (open_time + self._sec_interval):
                         self._prev_predict = utils.convert_signal_str(self.returns[-1])
-                        __now__ += self._sec_interval
+                        open_time += self._sec_interval
                         break
             except Exception as exc:
+                self.client.exit_last_order()
                 if ignore_exceptions:
                     if print_exc:
                         print(exc)
@@ -1548,7 +1479,7 @@ winrate: {self.winrate}%"""
         patterns = map(utils.anti_set_, patterns)
         return self.multi_strategy_collider(*patterns, mode='super')
 
-    def get_support_resistanse(self) -> Dict[str, Dict[int, float]]:
+    def get_support_resistance(self) -> Dict[str, Dict[int, float]]:
         lows = self.df['Low'].values
         highs = self.df['High'].values
         for i in range(2, len(lows) - 2):
