@@ -1,7 +1,7 @@
 import datetime as dt
 import json
 import logging
-from typing import Any, List, Union
+from typing import Any, List, Union, Tuple, Iterable
 
 import numpy as np
 import pandas as pd
@@ -26,7 +26,7 @@ __credits__: List[str] = ["Hemerson Tacon -- Stack overflow",
                           "hpaulj -- Stack overflow",
                           "furas -- Stack overflow",
                           "Devin Jeanpierre (edit: wjandrea) -- Stack overflow",
-                          "Войтенко Николай Поликарпович (Vojtenko Nikolaj Polikarpovich) -- helped me test the "
+                          "Войтенко Николай Поликарпович (Vojtenko Nikolay Polikarpovich) -- helped me test the "
                           "system of interaction with the binance crypto exchange with 50 dollars.",
 
                           "https://fxgears.com/index.php?threads/how-to-acquire-free-historical-tick-and-bar-data-for"
@@ -37,7 +37,7 @@ __credits__: List[str] = ["Hemerson Tacon -- Stack overflow",
                           "https://stackoverflow.com/questions/57838939/handling-exceptions-with-bulk-api-requests --"
                           "IEX token",
                           "Igor Kroitor -- donate 0.5 ETH (~1320$)"]
-__version__: str = "4.6.5"
+__version__: str = "4.7.0"
 
 SCATTER_SIZE: float = 12.0
 SCATTER_ALPHA: float = 1.0
@@ -49,8 +49,6 @@ TEXT_COLOR: str = 'white'
 SUB_LINES_WIDTH: float = 3.0
 STOP_TAKE_OPN_ALPHA: float = 0.8
 COLOR_DEPOSIT: str = 'white'
-DEPO_COLOR_UP: str = 'green'
-DEPO_COLOR_DOWN: str = 'red'
 
 logger = logging.getLogger()
 logger.setLevel(30)
@@ -167,7 +165,7 @@ def anti_set_(seted: List[Any], _nan_num: float = 18699.9) -> List[Any]:
     return ret
 
 
-def get_window(values, window_length: int) -> List[Any]:
+def get_window(values, window_length: int) -> List[Iterable[Any]]:
     ret: List[Any] = []
     for e, i in enumerate(values[:len(values) - window_length + 1]):
         ret.append(values[e:e + window_length])
@@ -231,3 +229,68 @@ def get_linear(dataset) -> np.ndarray:
         return_list.append(start + mean_diff * i)
     logger.debug(f'in linear: self.mean_diff={mean_diff}')
     return np.array(return_list)
+
+
+def get_coef_sec(timeframe: str = '1d') -> Tuple[float, int]:
+    profit_calculate_coef: float
+    sec_interval: int
+    if timeframe == '1m':
+        profit_calculate_coef = (60 * 24 * 365)
+        sec_interval = 60
+    elif timeframe == '2m':
+        profit_calculate_coef = (30 * 24 * 365)
+        sec_interval = 120
+    elif timeframe == '3m':
+        profit_calculate_coef = (20 * 24 * 365)
+        sec_interval = 180
+    elif timeframe == '5m':
+        profit_calculate_coef = (12 * 24 * 365)
+        sec_interval = 300
+    elif timeframe == '15m':
+        profit_calculate_coef = (4 * 24 * 365)
+        sec_interval = 15 * 60
+    elif timeframe == '30m':
+        profit_calculate_coef = (2 * 24 * 365)
+        sec_interval = 60 * 30
+    elif timeframe == '45m':
+        profit_calculate_coef = (32 * 365)
+        sec_interval = 60 * 45
+    elif timeframe == '1h':
+        profit_calculate_coef = (24 * 365)
+        sec_interval = 60 * 60
+    elif timeframe == '90m':
+        profit_calculate_coef = (18 * 365)
+        sec_interval = 60 * 90
+    elif timeframe == '2h':
+        profit_calculate_coef = (12 * 365)
+        sec_interval = 60 * 60 * 2
+    elif timeframe == '3h':
+        profit_calculate_coef = (8 * 365)
+        sec_interval = 60 * 60 * 3
+    elif timeframe == '4h':
+        profit_calculate_coef = (6 * 365)
+        sec_interval = 60 * 60 * 4
+    elif timeframe == '12h':
+        profit_calculate_coef = (2 * 365)
+        sec_interval = 60 * 60 * 12
+    elif timeframe == '1d':
+        profit_calculate_coef = 365
+        sec_interval = 60 * 60 * 24
+    elif timeframe == '3d':
+        profit_calculate_coef = (365 / 3)
+        sec_interval = 86400 * 3
+    elif timeframe == '1w':
+        profit_calculate_coef = 52
+        sec_interval = 86400 * 7
+    elif timeframe == '1M':
+        profit_calculate_coef = 12
+        sec_interval = 86400 * 30
+    elif timeframe == '3M':
+        profit_calculate_coef = 4
+        sec_interval = 86400 * 90
+    elif timeframe == '6M':
+        profit_calculate_coef = 2
+        sec_interval = 86400 * 180
+    else:
+        raise ValueError(f'incorrect interval; {timeframe}')
+    return profit_calculate_coef, sec_interval
